@@ -1,12 +1,11 @@
-import sys, re
+import re
 import numpy as np
+import basis_set_exchange as bse
 
 from enum import Enum
 
-from cctk import File, Ensemble, Molecule, ConformationalEnsemble, OneIndexedArray
-from cctk.helper_functions import get_symbol, compute_distance_between, \
-                                  compute_angle_between, compute_dihedral_between, \
-                                  get_number, get_corrected_free_energy
+from cctk import File, Molecule, ConformationalEnsemble, OneIndexedArray
+from cctk.helper_functions import get_symbol, get_number, get_corrected_free_energy
 
 import cctk.parse_gaussian as parse
 
@@ -71,7 +70,7 @@ class GaussianFile(File):
     Class representing Gaussian input/output files.
 
     Attributes:
-        ensemble (Ensemble): ``ConformationalEnsemble`` instance
+        ensemble (ConformationalEnsemble): ``ConformationalEnsemble`` instance
         job_types (list): list of `job_type`` instances
         route_card (str): optional, route card of .gjf file
         link0 (dict): optional, dictionary of Link 0 commands (e.g. {"mem": "32GB", "nprocshared": 16})
@@ -112,7 +111,7 @@ class GaussianFile(File):
         if success and not isinstance(success, int):
             raise TypeError("success needs to be an integer")
 
-        if not isinstance(elapsed_time, float) or elapsed_time < 0.0:
+        if not isinstance(elapsed_time, (float, int)) or elapsed_time < 0.0:
             raise TypeError(f"elapsed_time invalid: {elapsed_time}")
 
         if job_types is not None:
@@ -152,7 +151,8 @@ class GaussianFile(File):
         if (route_card is None) or (not isinstance(route_card, str)):
             raise ValueError("can't write a file without a route card")
 
-        assert re.match(r"^#p", route_card), f"route card doesn't start with #p: {route_card}"
+        if not re.match(r"^#p", route_card):
+            print(f"ALERT - route card doesn't start with #p: {route_card}")
 
         #### generate the text
         text = ""
@@ -231,6 +231,7 @@ class GaussianFile(File):
             return list()
 
     @classmethod
+#    @profile
     def read_file(cls, filename, return_lines=False, extended_opt_info=False):
         """
         Reads a Gaussian``.out`` or ``.gjf`` file and populates the attributes accordingly.
@@ -254,6 +255,7 @@ class GaussianFile(File):
 
         link1_lines = parse.split_link1(filename)
         files = []
+
         for link1idx, lines in enumerate(link1_lines):
             #### automatically assign job types based on header
             header = lines.search_for_block("#p", "----", format_line=lambda x: x.lstrip(), join="")
@@ -265,33 +267,20 @@ class GaussianFile(File):
 
             title = ""
             title_block = lines.search_for_block("l101.exe", "Symbolic Z-matrix", join="\n")
-            for line in title_block.split("\n")[1:]:
-                if not re.search("-----", line):
-                    title += line
+            if title_block is not None:
+                for line in title_block.split("\n")[1:]:
+                    if not re.search("-----", line):
+                        title += line
 
-            #### extract parameters
-            success = 0
-            elapsed_time = 0.0  # in seconds
-            for line in lines:
-                if line.strip().startswith("Normal termination"):
-                    success += 1
-                elif line.startswith(" Elapsed time:"):
-                    # Elapsed time:       0 days  0 hours  0 minutes  0.6 seconds.
-                    fields = line.split()
-                    assert len(fields) == 10, f"unexpected number of fields on elapsed time line:\n{line}"
-                    days = float(fields[2])
-                    hours = float(fields[4])
-                    minutes = float(fields[6])
-                    seconds = float(fields[8])
-                    elapsed_time += days * 86400 + hours * 3600 + minutes * 60 + seconds
 
-            (geometries, atom_list, energies, scf_iterations,) = parse.read_geometries_and_energies(lines)
+            (geometries, atom_list, energies, scf_iterations, success, elapsed_time) = parse.read_geometries_and_energies(lines)
+            success, elapsed_time = parse.extract_success_and_time(lines)
             atomic_numbers = []
 
             #### convert to right datatype
             try:
                 atomic_numbers = np.array(atom_list, dtype=np.int8)
-            except:
+            except Exception as e:
                 atomic_numbers = np.array(list(map(get_number, atom_list)), dtype=np.int8)
 
             footer = None
@@ -302,8 +291,7 @@ class GaussianFile(File):
                     footer = "\n".join([" ".join(list(filter(None, line.split(" ")))) for line in footer.split("\n")])
 
             bonds = parse.read_bonds(lines)
-            charge = lines.find_parameter("Multiplicity", expected_length=4, which_field=1, split_on="=")[0]
-            multip = lines.find_parameter("Multiplicity", expected_length=4, which_field=3, split_on="=")[0]
+            charge, multip =  lines.find_parameter("Multiplicity", expected_length=4, which_field=[1,3], split_on="=")[0]
 
             f = GaussianFile(job_types=job_types, route_card=header, link0=link0, footer=footer, success=success, elapsed_time=elapsed_time, title=title)
 
@@ -388,11 +376,9 @@ class GaussianFile(File):
 
                 frequencies = []
                 try:
-                    frequencies += lines.find_parameter("Frequencies", expected_length=5, which_field=2)
-                    frequencies += lines.find_parameter("Frequencies", expected_length=5, which_field=3)
-                    frequencies += lines.find_parameter("Frequencies", expected_length=5, which_field=4)
+                    frequencies = sum(lines.find_parameter("Frequencies", expected_length=5, which_field=[2,3,4]), [])
                     properties[-1]["frequencies"] = sorted(frequencies)
-                except:
+                except Exception as e:
                     raise ValueError("error finding frequencies")
 
                 #  Temperature   298.150 Kelvin.  Pressure   1.00000 Atm.
@@ -405,9 +391,14 @@ class GaussianFile(File):
 
 
             if GaussianJobType.NMR in job_types:
-                assert len(molecules) == 1, "NMR jobs should not be combined with optimizations!"
                 nmr_shifts = parse.read_nmr_shifts(lines, molecules[0].num_atoms())
-                properties[0]["isotropic_shielding"] = nmr_shifts.view(OneIndexedArray)
+                if nmr_shifts is not None:
+                    properties[-1]["isotropic_shielding"] = nmr_shifts.view(OneIndexedArray)
+
+                if re.search("nmr=mixed", f.route_card, flags=re.IGNORECASE) or re.search("nmr=spinspin", f.route_card,flags=re.IGNORECASE):
+                    couplings = parse.read_j_couplings(lines, molecules[0].num_atoms())
+                    if couplings is not None:
+                        properties[-1]["j_couplings"] = couplings
 
             if GaussianJobType.FORCE in job_types:
                 assert len(molecules) == 1, "force jobs should not be combined with optimizations!"
@@ -415,7 +406,7 @@ class GaussianFile(File):
                 properties[0]["forces"] = forces
 
             if GaussianJobType.POP in job_types:
-                if re.search("hirshfeld", f.route_card):
+                if re.search("hirshfeld", f.route_card) or re.search("cm5", f.route_card):
                     charges, spins = parse.read_hirshfeld_charges(lines)
                     properties[-1]["hirshfeld_charges"] = charges
                     properties[-1]["hirshfeld_spins"] = spins
@@ -423,13 +414,13 @@ class GaussianFile(File):
             try:
                 charges = parse.read_mulliken_charges(lines)
                 properties[-1]["mulliken_charges"] = charges
-            except:
+            except Exception as e:
                 pass
 
             try:
                 dipole = parse.read_dipole_moment(lines)
                 properties[-1]["dipole_moment"] = dipole
-            except:
+            except Exception as e:
                 pass
 
             for mol, prop in zip(molecules, properties):
@@ -523,7 +514,7 @@ class GaussianFile(File):
 
         try:
             atomic_numbers = np.array(atomic_numbers, dtype=np.int8)
-        except:
+        except Exception as e:
             atomic_numbers = np.array(list(map(get_number, atomic_numbers)), dtype=np.int8)
 
         job_types = cls._assign_job_types(header)
@@ -611,18 +602,16 @@ class GaussianFile(File):
             assert len(ensemble) > 0, "cannot write a blank ensemble"
 
             if isinstance(route_card, str):
-                assert re.match(r"^#p", route_card), "route card should start with #p: {route_card}"
-                route_card = [route_card for m in ensemble._items]
+                route_card = [route_card for _ in ensemble._items]
             elif isinstance(route_card, list):
                 assert len(route_card) == n_geometries, f"expected {n_geometries} route cards but got {len(route_card)}"
                 for card in route_card:
                     assert isinstance(card, str), "expected route card to be a str"
-                    assert re.match(r"^#p", route_card), "route card should start with #p: f{card}"
             else:
                 raise ValueError(f"unexpected type for route_card: {str(type(route_card))}")
 
             if isinstance(link0, dict):
-                link0 = [link0 for m in ensemble._items]
+                link0 = [link0 for _ in ensemble._items]
             elif isinstance(link0, list):
                 assert len(link0) == n_geometries, f"expected {n_geometries} link0 entries, but got {len(link0)}"
                 for d in link0:
@@ -631,7 +620,7 @@ class GaussianFile(File):
                 raise ValueError(f"unexpected type for link0: {str(type(link0))}")
 
             if footer is None or isinstance(footer, str):
-                footer = [footer for m in ensemble._items]
+                footer = [footer for _ in ensemble._items]
             elif isinstance(footer, list):
                 assert len(footer) == n_geometries, f"expected {n_geometries} footers, but got {len(footer)}"
                 for f in footer:
@@ -641,7 +630,7 @@ class GaussianFile(File):
 
             if isinstance(title, str):
                 assert len(title.strip()) > 0, "zero-length titles not allowed"
-                title = [title for m in ensemble._items]
+                title = [title for _ in ensemble._items]
             elif isinstance(title, list):
                 assert len(title) == n_geometries, f"expected {n_geometries} route cards but got {len(title)}"
                 for card in title:
@@ -651,7 +640,7 @@ class GaussianFile(File):
                 raise ValueError(f"unexpected type for title: {str(type(title))}")
 
             if isinstance(print_symbol, bool):
-                print_symbol = [print_symbol for m in ensemble._items]
+                print_symbol = [print_symbol for _ in ensemble._items]
             elif isinstance(print_symbol, list):
                 assert len(print_symbol) == n_geometries, f"expected {n_geometries} print_symbol entries but got {len(print_symbol)}"
                 for s in print_symbol:
@@ -665,4 +654,77 @@ class GaussianFile(File):
                 else:
                     cls.write_molecule_to_file(filename, molecule, route_card[idx], link0[idx], footer=footer[idx], title=title[idx], print_symbol=print_symbol[idx], append=True)
 
+    def add_custom_basis_set(self, name, add_all_elements=False, return_string=False):
+        """
+        Appends custom basis sets (from Basis Set Exchange) to ``self.footer``. Should be used in combination with the ``gen`` keyword.
+
+        Args:
+            name (str): name of basis set (look it up on Basis Set Exchange)
+            add_all_elements (bool): whether the complete basis set should be added or just the elements of interest
+            return_string (bool): if the basis set should be appended to the footer or returned as a string (no change to ``self``)
+
+        Returns:
+            nothing (if return_string is ``False``)
+            string of basis set definition (if return string is ``True``)
+        """
+        assert isinstance(name, str), "need basis set name to be a string, for starters"
+
+        try:
+            basis_definition = ""
+            if add_all_elements:
+                basis_definition = bse.get_basis(name, fmt="gaussian94", header=False)
+            else:
+                elements = list(np.unique(self.get_molecule().atomic_numbers.view(np.ndarray)))
+                basis_definition = bse.get_basis(name, fmt="gaussian94", header=False, elements=elements)
+
+            if self.footer is None:
+                self.footer = basis_definition
+            else:
+                self.footer += basis_definition
+            self.footer += "\n"
+
+        except Exception as e:
+            raise ValueError(f"adding basis set {name} from basis set exchange failed!\n{e}")
+
+    @classmethod
+    def read_file(cls, filename, return_lines=False, extended_opt_info=False):
+#    def read_fast(cls, filename, return_lines=False, extended_opt_info=False):
+        """
+        Reads a Gaussian``.out`` or ``.gjf`` file and populates the attributes accordingly.
+        Only footers from ``opt=modredundant`` can be read automatically --  ``genecep`` custom basis sets, &c must be specified manually.
+
+        Note:
+
+        Will throw ``ValueError`` if there have been no successful iterations.
+
+        Args:
+            filename (str): path to the out file
+            return_lines (Bool): whether the lines of the file should be returned
+            extended_opt_info (Bool): if full parameters about each opt step should be collected
+                (by default, only ``rms_displacement`` and ``rms_force`` are collected)
+        Returns:
+            ``GaussianFile`` object (or list of ``GaussianFile`` objects for Link1 files)
+            (optional) the lines of the file (or list of lines of file for Link1 files) as Lines object
+        """
+        if re.search("gjf$", filename) or re.search("com$", filename):
+            return cls._read_gjf_file(filename, return_lines)
+
+        link1_lines = parse.split_link1_to_text(filename)
+        files = []
+
+        for link1idx, lines in enumerate(link1_lines):
+            files.append(parse.read_file_fast(lines, filename, link1idx, extended_opt_info=extended_opt_info))
+
+        if return_lines:
+            link1_lines = parse.split_link1(filename)
+            if len(link1_lines) == 1:
+                return files[0], link1_lines[0]
+            else:
+                return files, link1_lines
+        else:
+            if len(link1_lines) == 1:
+                return files[0]
+            else:
+                return files
+
 
