@@ -4,7 +4,7 @@ Miscellaneous helper functions.
 
 import numpy as np
 import math
-import re
+from scipy.spatial.distance import cdist
 
 #### python 3.6 or earlier doesn't have importlib.resources, but it's backported as importlib_resources
 try:
@@ -251,55 +251,28 @@ def align_matrices(P_partial, P_full, Q_partial, return_matrix=False):
     rotation = U @ middle @ Vt
     return P_full @ rotation
 
-
-def compute_RMSD(geometry1, geometry2, using_atom_numbers="all", checks=True):
+def compute_RMSD(geometry1, geometry2, checks=True):
     """
-    Computes the root mean squared difference between two molecules/geometries.
+    Computes the root mean squared difference between two geometries.
 
     Args:
-        geometry1 (Molecule or np.array (dimensions: n atoms x 3): molecule or geometry
-        geometry2 (Molecule or np.array (dimensions: n atoms x 3): molecule or geometry
-        using_atom_numbers (str or list): which atoms to consider for the RMSD calculation;
-                                          use "all" to consider all atoms or a list of
-                                          1-indexed atom numbers
+        geometry1 (np.array (dimensions: n atoms x 3): geometry
+        geometry2 (np.array (dimensions: n atoms x 3): geometry
         checks (bool): whether to check that the inputs make sense (True by default)
 
     Returns:
         the root-mean-square distance between the two geometries
     """
-    if isinstance(geometry1, cctk.Molecule):
-        geometry1 = geometry1.geometry
-    if isinstance(geometry2, cctk.Molecule):
-        geometry2 = geometry2.geometry
     if checks and not isinstance(geometry1, cctk.OneIndexedArray):
         raise ValueError(f"expected cctk.OneIndexedArray but got {str(type(geometry1))} instead")
     if checks and not isinstance(geometry2, cctk.OneIndexedArray):
         raise ValueError(f"expected cctk.OneIndexedArray but got {str(type(geometry2))} instead")
 
-    n_atoms = len(geometry1)
-    if checks and len(geometry2) != n_atoms:
+    if checks and len(geometry2) != len(geometry1):
         raise ValueError("can't compare two geometries with different lengths!")
 
-    if checks:
-        if isinstance(using_atom_numbers, str):
-            if using_atom_numbers == "all":
-                using_atom_numbers = list(range(1,n_atoms+1))
-            else:
-                raise ValueError(f"unknown argument {using_atom_numbers}")
-        elif isinstance(using_atom_numbers, (list, np.ndarray)):
-            assert len(using_atom_numbers) > 0, "must specify at least one atom number"
-            n_atoms = len(geometry1)
-            for i in using_atom_numbers:
-                assert 1 <= i <= n_atoms, f"atom number {i} out of range"
-            assert len(set(using_atom_numbers)) == len(using_atom_numbers), f"duplicate atom numbers found"
-        else:
-            raise ValueError(f"unexpected type for using_atom_numbers, expected str, list, or np.ndarray but got {str(type(using_atom_numbers))}")
-
-    partial_geometry_1 = geometry1[using_atom_numbers]
-    partial_geometry_2 = geometry2[using_atom_numbers]
-    squared_difference = np.square(partial_geometry_1 - partial_geometry_2)
-    temp = np.sum(squared_difference) / n_atoms
-    return np.sqrt(temp)
+#    return np.trace(cdist(geometry1, geometry2)) / len(geometry1)
+    return np.sqrt( np.sum( ( geometry1 - geometry2 ) ** 2) / len(geometry1) )
 
 def get_isotopic_distribution(z):
     """
@@ -333,7 +306,7 @@ DEFAULT_NMR_SCALING_FACTORS = {
         "N" : (0.9776, 244.5626)
 }
 
-def scale_nmr_shifts(ensemble, symmetrical_atom_numbers=None, scaling_factors="default"):
+def scale_nmr_shifts(ensemble, symmetrical_atom_numbers=None, scaling_factors="default", property_name="isotropic_shielding"):
     """
     Apply linear scaling to isotropic shieldings to get chemical shifts.
     Shifts are calculated as (intercept-shielding)/slope.
@@ -350,6 +323,8 @@ def scale_nmr_shifts(ensemble, symmetrical_atom_numbers=None, scaling_factors="d
         scaling_factors: "default" to use DEFAULT_NMR_SCALING_FACTORS or a dict
                          (atomic symbol --> (slope,intercept)).  Elements for
                          which scaling factors are not provided will be ignored.
+        property_name:   the key in properties_dict to use to locate the predicted
+                         isotropic shieldings (default="isotropic_shielding")
 
     Returns:
         scaled_shifts: np.array (matching the shape of the original shieldings minus symmetry averaging)
@@ -368,19 +343,20 @@ def scale_nmr_shifts(ensemble, symmetrical_atom_numbers=None, scaling_factors="d
     else:
         assert isinstance(scaling_factors, dict)
         assert len(scaling_factors) > 0, "must provide scaling factors"
+    assert isinstance(property_name, str) and len(property_name)>0, f"property_name {property_name} is invalid"
 
     # get shieldings and scale
     all_scaled_shifts = []
     all_shift_labels = []
     for i,(molecule,properties) in enumerate(ensemble.items()):
-        if "isotropic_shielding" in properties:
+        if property_name in properties:
             # get atom numbers and atomic elements as OneIndexedArrays
             atomic_numbers = molecule.atomic_numbers
             n_atoms = len(atomic_numbers)
             atomic_symbols = [ get_symbol(n) for n in atomic_numbers ]
             atomic_symbols = cctk.OneIndexedArray(atomic_symbols)
             atom_numbers = list(range(1,n_atoms+1))
-            symbol_dict = dict(zip(atomic_numbers,atomic_symbols))
+#            symbol_dict = dict(zip(atomic_numbers,atomic_symbols))
             all_labels = [ f"{current_symbol}{atom_number}" for current_symbol,atom_number in zip(atomic_symbols,atom_numbers) ]
             all_labels = cctk.OneIndexedArray(all_labels)
             label_dict = dict(zip(atom_numbers,all_labels))
@@ -389,7 +365,7 @@ def scale_nmr_shifts(ensemble, symmetrical_atom_numbers=None, scaling_factors="d
             n_atoms = len(atomic_numbers)
             symmetrical_groups_dict = {}    # symbol --> [ [list1], [list2], ...] where each list is a group of symmetrical atom numbers
             symmetrical_groups_dict2 = {}   # symbol --> [ union of all symmetrical atom numbers for this symbol ]
-            unique_atoms_dict = {}          # symbol --> [ union of all unique atom numbers for this symbol ]
+#            unique_atoms_dict = {}          # symbol --> [ union of all unique atom numbers for this symbol ]
             for symmetrical_group in symmetrical_atom_numbers:
                 assert len(symmetrical_group) > 1, "must be at least 2 symmetrical nuclei in a group"
                 assert len(symmetrical_group) == len(set(symmetrical_group)), f"check for duplicate atom numbers in {symmetrical_group}"
@@ -411,7 +387,7 @@ def scale_nmr_shifts(ensemble, symmetrical_atom_numbers=None, scaling_factors="d
                 symmetrical_groups_dict2[symmetrical_symbol].extend(symmetrical_group)
 
             # get shieldings
-            all_shieldings = properties["isotropic_shielding"]
+            all_shieldings = properties[property_name]
 
             # iterate through requested elements
             molecule_shifts = []
@@ -505,7 +481,6 @@ def compute_chirality(v1, v2, v3, v4):
     assert (isinstance(v4, np.ndarray) and len(v4) == 3), "v4 needs to be a 3-element np.ndarray!"
 
     e1 = np.array([1, 0, 0])
-    e2 = np.array([0, 1, 0])
     e3 = np.array([0, 0, 1])
 
     # rotate v4 so that it's pointing back!
@@ -530,8 +505,6 @@ def compute_chirality(v1, v2, v3, v4):
     v3 = R2 @ v3
     v4 = R2 @ v4
 
-    theta2 = compute_angle_between(np.array([0, 0, v1[2]]), e3) # projection of v1 onto e3
-
     assert 1.0 > compute_angle_between(v4, e1), f"rotating v4 failed - 1.0 ≤ {compute_angle_between(v4, e1)}"
     assert 1.0 > compute_angle_between(e3, np.array([0, 0, v1[2]])), f"rotating v1 failed, - 1.0 ≤ {compute_angle_between(e3, np.array([0, 0, v1[2]]))}"
 
