@@ -1,9 +1,9 @@
-import sys, re, math, copy, scipy
+import math, copy, scipy
 import numpy as np
 import networkx as nx
+from scipy.spatial.distance import cdist
 
 import cctk
-from cctk import OneIndexedArray
 from cctk.helper_functions import (
     get_symbol,
     get_number,
@@ -42,49 +42,38 @@ class Molecule:
 
         ``bonds`` must be a list of edges (i.e. an n x 2 ``numpy`` array).
         """
-        if checks:
-            if len(atomic_numbers) != len(geometry):
-                raise ValueError("length of geometry and atomic_numbers does not match!")
+        if len(atomic_numbers) != len(geometry):
+            raise ValueError(f"length of geometry ({len(geometry)}) and atomic_numbers ({len(atomic_numbers)}) does not match!\n{atomic_numbers}\n{geometry}")
 
-            if not all(isinstance(z, np.int8) for z in atomic_numbers) or atomic_numbers.size == 0:
-                raise ValueError("invalid atom list")
+        try:
+            atomic_numbers = np.asarray(atomic_numbers, dtype=np.int8).view(cctk.OneIndexedArray)
+        except Exception as e:
+            raise ValueError("invalid atom list")
 
-            if len(geometry) == 0:
-                raise ValueError("invalid geometry list")
+        try:
+            geometry = np.array(geometry, dtype=np.float32).view(cctk.OneIndexedArray)
+        except Exception as e:
+            raise TypeError("geometry cannot be cast to ``np.ndarray`` of floats!")
 
-            try:
-                geometry = np.array(geometry)
-                geometry = geometry.astype(float)
-            except:
-                raise TypeError("geometry cannot be cast to ``np.ndarray`` of floats!")
-
-            if not all(all(isinstance(y, float) for y in x) for x in geometry):
-                raise TypeError("each element of self.geometry must be a 3-tuple")
-
-            if name and not isinstance(name, str):
+        if name is not None:
+            if not isinstance(name, str):
                 raise TypeError("name must be a string!")
 
-            for atom in atomic_numbers:
-                try:
-                    get_symbol(atom)
-                except ValueError:
-                    raise ValueError(f"unknwon atomic number {atom}")
+        if not isinstance(charge, int):
+            try:
+                charge = int(charge)
+            except Exception as e:
+                raise TypeError("charge must be integer or castable to integer!")
 
-            if not isinstance(charge, int):
-                try:
-                    charge = int(charge)
-                except:
-                    raise TypeError("charge must be integer or castable to integer!")
+        if not isinstance(multiplicity, int):
+            try:
+                multiplicity = int(multiplicity)
+            except Exception as e:
+                raise TypeError("multiplicity must be positive integer or castable to positive integer")
+        assert multiplicity > 0, "multiplicity must be positive"
 
-            if not isinstance(multiplicity, int):
-                try:
-                    multiplicity = int(multiplicity)
-                except:
-                    raise TypeError("multiplicity must be positive integer or castable to positive integer")
-            assert multiplicity > 0, "multiplicity must be positive"
-
-        self.atomic_numbers = atomic_numbers.view(OneIndexedArray)
-        self.geometry = geometry.view(OneIndexedArray)
+        self.atomic_numbers = atomic_numbers
+        self.geometry = geometry
 
         self.name = name
         self.multiplicity = multiplicity
@@ -94,15 +83,19 @@ class Molecule:
             self.bonds = bonds
         elif isinstance(bonds, (list,np.ndarray,nx.classes.reportviews.EdgeView)):
             if checks:
+                known_atomic_numbers = set()
                 for bond in bonds:
-                    assert isinstance(bond, (tuple,list,np.ndarray)), f"expected a 2-iterable for bond, got {type(bond)}"
                     assert len(bond)==2, f"unexpected number of atoms in bond, expected 2, got {len(bond)}"
-                    self._check_atom_number(bond[0])
-                    self._check_atom_number(bond[1])
+                    if bond[0] not in known_atomic_numbers:
+                        self._check_atom_number(bond[0])
+                        known_atomic_numbers.add(bond[0])
+                    if bond[1] not in known_atomic_numbers:
+                        self._check_atom_number(bond[1])
+                        known_atomic_numbers.add(bond[1])
+
             self.bonds = nx.Graph()
             self.bonds.add_nodes_from(range(1, len(atomic_numbers) + 1))
-            for bond in bonds:
-                self.add_bond(bond[0], bond[1])
+            self.bonds.add_edges_from(bonds, weight=1)
         elif bonds is None:
             self.bonds = nx.Graph()
             self.bonds.add_nodes_from(range(1, len(atomic_numbers)+1))
@@ -115,12 +108,9 @@ class Molecule:
         else:
             return f"Molecule ({len(self.atomic_numbers)} atoms)"
 
-    def __hash__(self):
-        return hash(id(self))
-
-    def assign_connectivity(self, cutoff=0.1):
+    def assign_connectivity(self, cutoff=0.2, periodic_boundary_conditions=None):
         """
-        Automatically recalculates bonds based on covalent radii. If two atoms are closer than the sum of their covalent radii + 0.5 Angstroms, then they are considered bonded.
+        Automatically recalculates bonds based on covalent radii. If two atoms are closer than the sum of their covalent radii + ``cutoff`` Angstroms, then they are considered bonded.
 
         Args:
             cutoff (float): the threshold (in Angstroms) for how close two covalent radii must be to be considered bonded
@@ -129,19 +119,50 @@ class Molecule:
             self
         """
 
+        #### delete all edges
+        self.bonds = nx.create_empty_copy(self.bonds)
+
         assert isinstance(cutoff, (float, int)), "need cutoff to be numeric!"
+        g = self.geometry.view(np.ndarray)
+
+        dist_matrix = None
+
+        #### cdist is SO FAST
+        if periodic_boundary_conditions is None:
+            dist_matrix = cdist(g, g, "euclidean")
+        else:
+            # even 16 cdist calls is faster than any other implementation, i tested it
+            pbc = periodic_boundary_conditions
+            assert isinstance(pbc, np.ndarray) and len(pbc) == 3, "Need 3-element ``np.ndarray`` for PBCs"
+
+            nearby_cells = [
+                [0, 0, 0],
+                [pbc[0], 0, 0],
+                [0, pbc[1], 0],
+                [0, 0, pbc[2]],
+                [pbc[0], pbc[1], 0],
+                [pbc[0], 0, pbc[2]],
+                [0, pbc[1], pbc[2]],
+                [pbc[0], pbc[1], pbc[2]],
+            ]
+
+            dist_matrices = [cdist(g, g + np.array(nc), "euclidean") for nc in nearby_cells]
+            dist_matrices += [cdist(g, g - np.array(nc), "euclidean") for nc in nearby_cells]
+            distances_3d = np.stack(dist_matrices)
+            dist_matrix = distances_3d.min(axis=0)
+
+        covalent_radii = {z: get_covalent_radius(z) for z in set(self.atomic_numbers)}
+        radii_by_num = [covalent_radii[z] for z in self.atomic_numbers]
 
         for i in range(1, self.num_atoms() + 1):
+            r_i = radii_by_num[i-1]
             for j in range(i + 1, self.num_atoms() + 1):
-                distance = self.get_distance(i, j)
-                r_i = get_covalent_radius(self.get_atomic_number(i))
-                r_j = get_covalent_radius(self.get_atomic_number(j))
+                distance = dist_matrix[i-1][j-1]
+                r_j = radii_by_num[j-1]
 
                 # 0.5 A distance is used by RasMol and Chime (documentation available online) and works well, empirically
                 if distance < (r_i + r_j + cutoff):
                     self.add_bond(i, j)
-                elif self.get_bond_order(i, j):
-                    self.remove_bond(i, j)
 
         return self
 
@@ -184,7 +205,7 @@ class Molecule:
 
         return True
 
-    def add_bond(self, atom1, atom2, bond_order=1):
+    def add_bond(self, atom1, atom2, bond_order=1, check=True):
         """
         Adds a new bond to the bond graph, or updates the existing bond order. Will not throw an error if the bond already exists.
 
@@ -193,11 +214,11 @@ class Molecule:
             atom2 (int): the number of the second atom
             bond_order (int): bond order of bond between atom1 and atom2
         """
-        self._check_atom_number(atom1)
-        self._check_atom_number(atom2)
-
-        assert isinstance(bond_order, int), f"bond order {bond_order} must be an integer"
-        assert bond_order >= 0, f"bond order {bond_order} must be positive"
+        if check:
+            self._check_atom_number(atom1)
+            self._check_atom_number(atom2)
+            assert isinstance(bond_order, int), f"bond order {bond_order} must be an integer"
+            assert bond_order >= 0, f"bond order {bond_order} must be positive"
 
         if self.bonds.has_edge(atom1, atom2):
             if bond_order == 0:
@@ -218,14 +239,8 @@ class Molecule:
         """
         Helper method which performs quick checks on the validity of a given atom number.
         """
-        if not isinstance(number, int):
-            raise TypeError("atom number must be integer")
-
-        if number > self.num_atoms():
-            raise ValueError(f"atom number {number} too large!")
-
-        if number <= 0:
-            raise ValueError(f"atom number {number} invalid: must be a positive integer!")
+        assert isinstance(number, int), "atomic number must be integer"
+        assert 0 < number <= self.num_atoms(), "atom number {number} too large! (or too small - needs to be >0)"
 
     def formula(self, return_dict=False):
         """
@@ -275,7 +290,7 @@ class Molecule:
 
     def _get_bond_fragments(self, atom1, atom2):
         """
-        Returns the pieces of a molecule that one would obtain by breaking the bond between two atoms. Will throw ``ValueError`` if the atoms are in a ring.
+        Returns the pieces of a molecule that one would obtain by ereaking the bond between two atoms. Will throw ``ValueError`` if the atoms are in a ring.
         Useful for distance/angle/dihedral scans -- one fragment can be moved and the other held constant.
 
         Args:
@@ -334,7 +349,6 @@ class Molecule:
         for fragment in fragments:
             if atom in fragment:
                 return list(fragment)
-                break
 
     def set_distance(self, atom1=None, atom2=None, distance=None, move="group", atoms=None):
         """
@@ -380,6 +394,9 @@ class Molecule:
         else:
             raise ValueError(f"Invalid option {move} for parameter 'move'!")
 
+        if (atom1 in atoms_to_move and atom2 in atoms_to_move) and move == "group":
+            raise ValueError('both our atoms are connected which will preclude any movement with ``move`` set to "group"')
+
         current_distance = self.get_distance(atom1, atom2)
 
         v1 = self.get_vector(atom1)
@@ -450,7 +467,7 @@ class Molecule:
 
         try:
             angle = float(angle)
-        except:
+        except Exception as e:
             raise TypeError(f"angle {angle} cannot be converted to float!")
 
         if (not isinstance(angle, float)) or ((angle < 0) or (angle > 360)):
@@ -567,7 +584,7 @@ class Molecule:
 
         try:
             dihedral = float(dihedral)
-        except:
+        except Exception as e:
             raise TypeError(f"dihedral angle {dihedral} cannot be converted to float!")
 
         if (not isinstance(dihedral, float)) or ((dihedral < 0) or (dihedral > 360)):
@@ -632,8 +649,6 @@ class Molecule:
         for atom in atoms_to_move:
             self.geometry[atom] = np.dot(rot_matrix, self.get_vector(atom, check=False))
 
-        error = self.get_dihedral(atom1, atom2, atom3, atom4, check=False) - dihedral
-
         #### and move it back!
         self.translate_molecule(v3)
 
@@ -657,8 +672,10 @@ class Molecule:
         Returns:
             the Molecule object
         """
-        for atom in range(1, self.num_atoms() + 1):
-            self.geometry[atom] = self.geometry[atom] + vector
+#        for atom in range(1, self.num_atoms() + 1):
+#            self.geometry[atom] = self.geometry[atom] + vector
+
+        self.geometry += vector
 
         return self
 
@@ -796,8 +813,8 @@ class Molecule:
             raise TypeError(f"symbol {symbol} must be a string!")
 
         number = get_number(symbol)
-        self.atomic_numbers = np.append(self.atomic_numbers, [number]).astype(np.int8).view(OneIndexedArray)
-        self.geometry = np.append(self.geometry, [coordinates], axis=0).view(OneIndexedArray)
+        self.atomic_numbers = np.append(self.atomic_numbers, [number]).astype(np.int8).view(cctk.OneIndexedArray)
+        self.geometry = np.append(self.geometry, [coordinates], axis=0).view(cctk.OneIndexedArray)
         self.bonds.add_node(self.num_atoms())
 
         return self
@@ -817,10 +834,14 @@ class Molecule:
 
         try:
             self.bonds.remove_node(number)
-            self.geometry = np.delete(self.geometry, number - 1, axis=0).view(OneIndexedArray)
-            self.atomic_numbers = np.delete(self.atomic_numbers, number - 1).view(OneIndexedArray)
+            self.geometry = np.delete(self.geometry, number - 1, axis=0).view(cctk.OneIndexedArray)
+            self.atomic_numbers = np.delete(self.atomic_numbers, number - 1).view(cctk.OneIndexedArray)
+
+            #### need to renumber to fill gaps
+            self.bonds = nx.convert_node_labels_to_integers(self.bonds, first_label=1, ordering="sorted")
+
             return self
-        except:
+        except Exception as e:
             raise ValueError("removing atom {number} failed!")
 
     def get_atomic_number(self, atom):
@@ -858,7 +879,7 @@ class Molecule:
         """
         n_atoms = self.get_n_atoms()
         l = [ self.get_atomic_symbol(i) for i in range(1,n_atoms+1) ]
-        return OneIndexedArray(l)
+        return cctk.OneIndexedArray(l)
 
     def get_n_atoms(self):
         """
@@ -918,7 +939,7 @@ class Molecule:
             try:
                 atom1 = int(atom1)
                 atom2 = int(atom2)
-            except:
+            except Exception as e:
                 raise TypeError("atom numbers cannot be cast to int!")
 
             self._check_atom_number(atom1)
@@ -942,7 +963,7 @@ class Molecule:
             try:
                 atom1 = int(atom1)
                 atom2 = int(atom2)
-            except:
+            except Exception as e:
                 raise TypeError("atom numbers cannot be cast to int!")
 
             self._check_atom_number(atom1)
@@ -979,7 +1000,7 @@ class Molecule:
                 atom1 = int(atom1)
                 atom2 = int(atom2)
                 atom3 = int(atom3)
-            except:
+            except Exception as e:
                 raise TypeError("atom numbers cannot be cast to int!")
 
             self._check_atom_number(atom1)
@@ -1024,7 +1045,7 @@ class Molecule:
                 atom2 = int(atom2)
                 atom3 = int(atom3)
                 atom4 = int(atom4)
-            except:
+            except Exception as e:
                 raise TypeError("atom numbers cannot be cast to int!")
 
             self._check_atom_number(atom1)
@@ -1104,7 +1125,7 @@ class Molecule:
         """
         try:
             atom = int(atom)
-        except:
+        except Exception as e:
             raise TypeError(f"atom number {atom} cannot be cast to int!")
 
         self._check_atom_number(atom)
@@ -1173,7 +1194,7 @@ class Molecule:
         """
         try:
             atom = int(atom)
-        except:
+        except Exception as e:
             raise ValueError("atom cannot be cast to int")
 
         self._check_atom_number(atom)
@@ -1221,8 +1242,8 @@ class Molecule:
             new ``Molecule`` object
         """
 
-        atoms = np.hstack((molecule1.atomic_numbers.T, molecule2.atomic_numbers.T)).view(OneIndexedArray)
-        geoms = np.vstack((molecule1.geometry, molecule2.geometry)).view(OneIndexedArray)
+        atoms = np.hstack((molecule1.atomic_numbers.T, molecule2.atomic_numbers.T)).view(cctk.OneIndexedArray)
+        geoms = np.vstack((molecule1.geometry, molecule2.geometry)).view(cctk.OneIndexedArray)
         charge = molecule1.charge + molecule2.charge
 
         s1 = (molecule1.multiplicity - 1) / 2
@@ -1348,15 +1369,15 @@ class Molecule:
         if isinstance(check_chirality, list):
             #### find all the differences and exchange them
             model_report = model.get_chirality_report(check_chirality)
-            mol_report = mol.get_chirality_report(check_chirality)
 
             #### generate all meso ring permutations
             candidates = mol.flip_meso_rings(atoms=check_chirality)
 
             #### for each, try flipping configuration of all centers
             for candidate in candidates:
+                report = candidate.get_chirality_report(check_chirality)
                 for center in check_chirality:
-                    if model_report[center] != candidate.get_chirality_report(check_chirality)[center]:
+                    if model_report[center] != report[center]:
                         try:
                             candidate = candidate.exchange_identical_substituents(center)
                         except ValueError as e:
@@ -1394,7 +1415,7 @@ class Molecule:
                 mol = self.exchange_identical_substituents(center)
                 exchangeable_centers.append(center)
                 continue
-            except:
+            except Exception as e:
                 pass
 
             mols = self.flip_meso_rings(atoms=[center])
@@ -1447,6 +1468,7 @@ class Molecule:
         if self_permutations is not None:
             returns = self_permutations
 
+
         for i in range(len(neighbors)):
             for j in range(i+1, len(neighbors)):
                 try:
@@ -1458,9 +1480,7 @@ class Molecule:
 
                     nm = nx.algorithms.isomorphism.categorical_node_match("atomic_number", 0)
                     match = nx.algorithms.isomorphism.GraphMatcher(graph1, graph2, node_match=nm)
-
                     if match.is_isomorphic():
-                        new_returns = []
                         for mol in returns:
                             new_mol = copy.deepcopy(mol)
                             for k,v in match.mapping.items():
@@ -1512,7 +1532,6 @@ class Molecule:
                     continue
 
                 #### reorder to put ``center`` first
-                idx = cycle.index(center)
                 while cycle[0] != center:
                     # why yes, this /is/ a O(n) solution for reordering a list. why do you ask?
                     cycle = cycle[1:] + cycle[0:1]
@@ -1571,3 +1590,80 @@ class Molecule:
                 returns = returns + new_returns
         return returns
 
+    def get_components(self):
+        """
+        Returns a list of all the connected components in a molecule.
+        """
+        assert self.bonds.number_of_edges() > 0, "need a bond graph to perform this operation -- try calling self.assign_connectivity()!"
+        fragments = nx.connected_components(self.bonds)
+        return [list(f) for f in list(fragments)]
+
+    def limit_solvent_shell(self, num_atoms=0, num_solvents=10):
+        """
+        Automatically detects solvent molecules and removes them until you have a set number of solvents or atoms.
+
+        This code assumes that the fragment of interest comes first in the file, which is almost always true but may not be true.
+
+        Args:
+            num_atoms (int): remove atoms until there are this number (module size of a solvent molecule)
+            num_solvents (int): remove solvent molecules until there are this number
+
+        Returns:
+            new ``Molecule`` object
+        """
+        assert isinstance(num_atoms, int)
+        assert isinstance(num_solvents, int)
+
+        fragments = self.get_components()
+
+        #### not strictly a centroid since it's not mass-weighted.
+        centroids = np.zeros(shape=(len(fragments), 3))
+        distances = np.zeros(shape=len(fragments))
+        for i, f in enumerate(fragments):
+            centroids[i] = np.mean(self.geometry[f], axis=0)
+            if i > 0:
+                distances[i] = np.linalg.norm(centroids[i] - centroids[0])
+
+        mol = copy.deepcopy(self)
+        #### reverse order - farthest away comes first
+        order = np.argsort(distances)[::-1]
+
+        current_num_solvents = len(fragments) - 1
+        current_num_atoms = mol.num_atoms()
+
+        to_remove = []
+        for i in order:
+            for j in fragments[i]:
+                to_remove.append(j)
+                current_num_atoms += -1
+            current_num_solvents += -1
+
+            if current_num_atoms <= num_atoms or num_solvents == current_num_solvents:
+                #### have to remove in reverse direction for indexing consistency
+                for j in sorted(to_remove, reverse=True):
+                    mol.remove_atom(j)
+                return mol
+
+    def center_periodic(self, center, side_length):
+        """
+        Adjusts a molecule to be in the center of a cube, moving all other molecules accordingly. Bonded subgroups will be moved as a unit.
+
+        For analysis of MD files with periodic boundary conditions.
+
+        Args:
+            center (int): atomic number to center
+            side_length (float): length of side, in Å
+        """
+        self._check_atom_number(center)
+        assert isinstance(side_length, (int, float))
+        assert side_length > 0
+
+        #### Center the atom of interest
+        self.geometry += -1 * self.geometry[center]
+        self.geometry += side_length / 2
+
+        for f in self.get_components():
+            centroid = np.mean(self.geometry[f], axis=0)
+            self.geometry[f] += -1 * np.floor_divide(centroid, side_length) * side_length
+
+        return self
