@@ -1,19 +1,19 @@
-import sys, re
+import re
 import numpy as np
 
 from enum import Enum
 #from abc import abstractmethod
 
 from cctk import File, Molecule, ConformationalEnsemble
-from cctk.helper_functions import get_symbol, compute_distance_between, compute_angle_between, compute_dihedral_between, get_number, get_corrected_free_energy
+from cctk.helper_functions import get_symbol, get_corrected_free_energy
 
 import cctk.parse_orca as parse
 
 class OrcaJobType(Enum):
     """
-    Class representing allowed Gaussian job types. Not an exhaustive list, but should be fairly comprehensive.
+    Class representing allowed Orca job types. Not an exhaustive list, but should be fairly comprehensive.
 
-    The value should be the Gaussian keyword, to permit automatic assignment.
+    The value should be the Orca keyword, to permit automatic assignment.
 
     All jobs have type ``SP`` by default.
     """
@@ -143,6 +143,11 @@ class OrcaFile(File):
                 properties[idx]["iteration"] = idx
                 properties[idx]["scf_iterations"] = iters[idx]
 
+            if multip > 1:
+                s2 = lines.find_parameter("Expectation value of", 6, 5)
+                for idx, spin_contam in enumerate(s2):
+                    properties[idx]["S**2"] = spin_contam
+
             if OrcaJobType.OPT in job_types:
                 rms_grad, max_grad, rms_step, max_step = parse.read_gradients(lines, len(properties))
                 for idx in range(len(rms_grad)):
@@ -185,20 +190,20 @@ class OrcaFile(File):
                 charges = parse.read_mulliken_charges(lines)
                 assert len(charges) == len(atomic_numbers)
                 properties[-1]["mulliken_charges"] = charges
-            except:
+            except Exception as e:
                 pass
 
             try:
                 charges = parse.read_loewdin_charges(lines)
                 assert len(charges) == len(atomic_numbers)
                 properties[-1]["lowdin_charges"] = charges
-            except:
+            except Exception as e:
                 pass
 
             try:
                 dipole = lines.find_parameter("Magnitude \(Debye\)", 4, 3)
                 properties[-1]["dipole_moment"] = dipole[0]
-            except:
+            except Exception as e:
                 pass
 
             for mol, prop in zip(molecules, properties):
@@ -328,7 +333,7 @@ class OrcaFile(File):
     @classmethod
     def _assign_job_types(cls, header):
         """
-        Assigns ``OrcaJobType`` objects from route card. ``GaussianJobType.SP`` is assigned by default.
+        Assigns ``OrcaJobType`` objects from route card. ``OrcaJobType.SP`` is assigned by default.
 
         Args:
             header (str): Orca header
