@@ -129,7 +129,9 @@ class TestMolecule(unittest.TestCase):
         v = np.array([1.5234,1.231234,-1.77777])
         mol = mol.translate_molecule(v)
 
-        self.assertListEqual(mol.geometry.tolist()[0], list(v))
+        for target, actual in zip(mol.geometry.tolist()[0], list(v)):
+            self.assertTrue(abs(target-actual) < 0.00001)
+#        self.assertListEqual(mol.geometry.tolist()[0], list(v))
         self.assertTrue(isinstance(mol.geometry, cctk.OneIndexedArray))
 
         mol2 = self.load_molecule()
@@ -150,7 +152,7 @@ class TestMolecule(unittest.TestCase):
 
     def test_volume(self):
         mol = self.load_molecule()
-        self.assertEqual(mol.volume(), 80.42662712363737)
+        self.assertEqual(mol.volume(), 80.42662516224424)
 
     def test_renumber(self):
         mol = self.load_molecule()
@@ -169,5 +171,21 @@ class TestMolecule(unittest.TestCase):
 
         mol.get_chirality_report()
 
+    def test_periodic_boundary_conditions(self):
+        m1 = cctk.Molecule([9, 9], [[0, 5, 4.5], [0, 0, 0.5]], charge=0, multiplicity=1)
+
+        m1.assign_connectivity()
+        self.assertFalse(m1.get_bond_order(1,2))
+
+        m1.assign_connectivity(periodic_boundary_conditions=np.array([5,5,5]))
+        self.assertTrue(m1.get_bond_order(1,2))
+
+        m2 = cctk.GaussianFile.read_file("test/static/periodic.gjf").get_molecule()
+        m2 = m2.assign_connectivity(periodic_boundary_conditions=np.array([20, 20, 20]))
+        m2.center_periodic(1, 20)
+
+        m3 = m2.limit_solvent_shell(num_solvents=10)
+        self.assertEqual(m3.num_atoms(), 83)
+
 if __name__ == '__main__':
     unittest.main()
